@@ -1,10 +1,17 @@
 __author__ = "jerryzhujian9@gmail.com"
 
 __doc__ = """This module is for easy interaction with linux, Mac OS X, Windows shell.
-Usage:
-1) import
+=============================================
+jerryzhujian9_at_gmail.com
+Tested under python 2.7
+To see your python version
+in terminal: python -V
+or in python: import sys; print (sys.version)
+=============================================
+Install:
+https://pypi.python.org/pypi/ez
+pip install ez
 
-2) type commands
 Almost all commands support the usage of '~', '..', '.', '?', '*' in path (ls,fls only support regular expression).
 Symbolic link itself is the target of file operations; the actual file should be safe.
 
@@ -15,6 +22,7 @@ pwd() or cwd()  # Returns current working director.
 csd(), csf()   # Returns current script directory, i.e. the directory where the running script is.
 parentdir(path) # Returns the parent directory of a path.
 joinpath(path1[, path2[, ...]])   # Returns the joined path. Supports vectorization.
+join(sep,string1,string2) # Glues together strings with sep. Supports vectorization.
 splitpath(path) # Returns a list of path elements: [path, file, ext]. Supports vectorization.
 cd(path)    # Changes to a new working directory.
 
@@ -46,6 +54,7 @@ help(name)/doc(name) # name is a string, Prints the doc string of a module/class
         three double quotes Returns, blabla three double quotes      <-----this is function's doctoring, use implicit
         return sth
 ver(package_name) version(package_name), see a package's version.  package_name could be 'python'
+whos(name),whos() list imported functions/packages
 
 log(file="log.txt", mode='a', status=True) # Prints output to both terminal and a file (log.txt) globally. mode: a=append; w=overwrite
 
@@ -61,10 +70,11 @@ clear(module, recursive=False)
 
 num(string)
 isempty(s)
-Randomize(x) # Sets a randomization seed.
-RandomizeArray(list=[])    # Shuffles a list in place.
-Random(a,b) # Returns a random integer N such that a <= N <= b.
-RandomChoice(seq) # Returns a random element from sequence
+Randomize(x), randomize(x) # Sets a randomization seed.
+RandomizeArray(list=[])   randomizearray(list=[])  # Shuffles a list in place.
+Random(a,b) random(a,b) # Returns a random integer N such that a <= N <= b.
+RandomChoice(seq), randomchoice(seq) # Returns a random element from sequence
+Permute(iterable=[]) permute(iterable=[]) # Returns permutations in a list
 
 unique(seq), union(seq1,seq2), intersect(seq1,seq2), setdiff(seq1,seq2) in original order
     note: setdiff(seq1,seq2) may not be equal to setdiff(seq2,seq1)
@@ -82,8 +92,13 @@ duplicate(seq) # returns a list of duplicated elements in original order
 JDict() # Jerry's dictionary, customized ordered dictionary class with convient attributes and methods, see help(JDict)
 Moment(timezone)    # Generates the current datetime in specified timezone, or local naive datetime if omitted.
 
-SetClip(content)   # Copy/Write something to current clipboard
-content = GetClip()   # Read out content from current clipboard and assign to a variable
+SetClip(content), setclip(content)   # Copy/Write something to current clipboard
+content = GetClip(), content = getclip()   # Read out content from current clipboard and assign to a variable
+
+lines(path='.', pattern='\.py$|.ini$|\.c$|\.h$|\.m$', recursive=True) # Counts lines of codes, counting empty lines as well.
+keygen(length=8, complexity=3)  # generate a random key
+
+isemailvalid(email) # True or False, isEmailValid, IsEmailValid
 """
 
 # reference: abspath for ../ ./, expanduser for ~, glob to resolve wildcards, fnmatch.translate wildcards to re
@@ -166,6 +181,39 @@ def joinpath(*args):
     else:
         return os.path.join(*args)
 
+def join(sep='',*args):
+    """join(sep,string1,string2)
+       Glues together strings with sep. Supports vectorization
+       Returns the joined string, supports vectorization
+    """
+    assert len(args) >= 1, "Give me more inputs"
+    vectorization = False
+    # to see whether there is an list/tuple input
+    for arg in args:
+        if type(arg) in [list, tuple]:
+            vectorization = True
+            length = len(arg)
+            break
+    if vectorization:
+        # fill arg which is a string
+        # args is by default an unmutable tuple
+        args = list(args)
+        for j in range(0,len(args)):
+            arg = args[j]
+            if type(arg) in [str]:
+                args[j] = [arg]*length
+        
+        results = []
+        for i in range(0,length):
+            # a bit tricy because of result + = result and .join
+            result = args[0][i]
+            for j in range(1,len(args)):
+                result = sep.join([result, args[j][i]])
+            results.append(result)
+        return results
+    else:
+        return sep.join(args)
+
 def splitpath(path):
     """splitpath(path), Split path into [dir, file, ext]. e.g., file=easyshell, ext=.py
     Supports vectorization."""
@@ -577,10 +625,29 @@ def ver(package_name='python'):
     if package_name == 'python': 
         print (sys.version)
     else:
-        # eval('import ' + package_name)
-        print eval(package_name + '.__version__')
+        theNameSpace = {}
+        exec('import ' + package_name, theNameSpace)
+        print theNameSpace[package_name].__version__
 version = ver
 
+def who(name=''):
+    """
+    whos(name),whos() list imported functions/packages
+    """
+    import inspect
+    caller = inspect.currentframe().f_back
+    theList = eval('dir('+ name + ')', caller.f_locals)
+    theList = sorted(theList)
+    theNameSpace = name if name != '' else 'Current'
+    print theNameSpace + ' namespace has the following existing functions/modules:'
+    # theString = ''
+    # for i in range(0,len(theList)):
+    #     sep = '\n' if i % 3 else '\t'
+    #     theString = theString + theList[i] + sep
+    theString = '\n'.join(theList)
+    print theString
+whos = who
+
 # def sedawk(path, regex=".*", search=None, replace=None, recursion=True):
 #     """Searches and replaces the matched content in every file with regular expression."""
 #     path = fullpath(path)
@@ -830,9 +897,21 @@ import os, sys, platform, string, random, re
 import datetime
 
 from random import seed as Randomize
+from random import seed as randomize
 from random import shuffle as RandomizeArray
+from random import shuffle as randomizearray
 from random import randint as Random
+from random import randint as random
 from random import choice as RandomChoice
+from random import choice as randomchoice
+def Permute(iterable=[]):
+  """Permute(iterable=[])
+  Returns permutations in a list
+  e.g., ([1,2,3]) --> [(1, 2, 3), (1, 3, 2), (2, 1, 3), (2, 3, 1), (3, 1, 2), (3, 2, 1)]
+  """
+  from itertools import permutations
+  return list(permutations(iterable))
+permute = Permute
 
 from time import sleep
 
@@ -1029,6 +1108,9 @@ class Moment(object):
     Formats the datetime to a string.
     Converts a datetime-like string to a Moment object.
 
+    Methods:
+    Shift, Format, Transform
+
     Attributes:
         moment, returns the datetime instance
 
@@ -1045,6 +1127,24 @@ class Moment(object):
         minute
         second
         microsecond
+
+    def __init__(self, timezone=None, moment=None):
+        Generates the current datetime in specified timezone, or local naive datetime if omitted, (when moment omitted).
+        Args:
+            timezone is the specified timezone string
+                US timezone:
+                US/Alaska, US/Aleutian, US/Arizona, US/Central, US/East-Indiana, US/Eastern,
+                US/Hawaii, US/Indiana-Starke, US/Michigan, US/Mountain, US/Pacific, US/Pacific-New, US/Samoa
+
+                Use the codes to look up:
+                from pytz import all_timezones
+                print len(all_timezones)
+                for zone in all_timezones:
+                    if 'US' in zone:
+                        print zone
+
+            moment could be naive (without tzinfo) or aware (with tzinfo) datetime
+
     """
     def __init__(self, timezone=None, moment=None):
         """Generates the current datetime in specified timezone, or local naive datetime if omitted, (when moment omitted).
@@ -1174,7 +1274,208 @@ class Moment(object):
 
 
 from pyperclip import copy as SetClip
+from pyperclip import copy as setclip
 from pyperclip import paste as GetClip
+from pyperclip import paste as getclip
+
+
+def lines(path='.', pattern='\.py$|.ini$|\.c$|\.h$|\.m$', recursive=True):
+    """Counts lines of codes, counting empty lines as well.
+    lines(path='.', pattern='\.py$|.ini$|\.c$|\.h$|\.m$', recursive=True)
+    """
+
+    # modified from https://liangsun.org/posts/python-code-for-counting-loclines-of-code/
+    # Created by Liang Sun <i@liangsun.org> in 2012
+    # This code is for Python 2.x
+    COUNT_EMPTY_LINE = True
+
+    def read_line_count(fname):
+        count = 0
+        for line in open(fname).readlines():
+            if COUNT_EMPTY_LINE or len(line.strip()) > 0:
+                count += 1
+        return count
+
+    line_count = 0
+    file_count = 0
+    files = fls(path,pattern) if recursive else ls(path,pattern)
+    for file in files:
+        try:
+            file_count += 1
+            c = read_line_count(file)
+            print "%s : %d" % (os.path.basename(file), c)
+            line_count += c
+        except:
+            pass
+    
+    print '-----------------------------'
+    print 'File counted: %d' % file_count
+    print 'Line counted: %d' % line_count
+    print 'Done!'
+
+def keygen(length=8, complexity=3):
+    """generate a random key
+    keygen(length=8, complexity=3)
+
+    length: how long is the generated key
+
+    complexity:
+    1 = digits only
+    2 = digits + lowercase  i.e., 'abcdefghijklmnopqrstuvwxyz'
+    3 = digits + uppercase (default)
+    4 = digits + lowercase + uppercase
+    5 = digits + lowercase + uppercase + punctuation
+    6 = digits + lowercase + uppercase + punctuation + whitespace
+    or alternatively, complexity could be a list of chars like, '0123456789abcdefABCDEF'
+    in which case, the key will only be composed of chars from the list
+    """
+    import string
+    import random
+    if complexity == 1:
+        chars = string.digits
+    elif complexity == 2:
+        chars = string.digits + string.ascii_lowercase
+    elif complexity == 3:
+        chars = string.digits + string.ascii_uppercase
+    elif complexity == 4:
+        chars = string.digits + string.ascii_lowercase + string.ascii_uppercase
+    elif complexity == 5:
+        chars = string.digits + string.ascii_lowercase + string.ascii_uppercase + string.punctuation
+    elif complexity == 6:
+        chars = string.digits + string.ascii_lowercase + string.ascii_uppercase + string.punctuation + string.whitespace
+    elif complexity not in [1, 2, 3, 4, 5, 6]:
+        chars = str(complexity)
+    return ''.join(random.choice(chars) for x in range(length))
+
+def isemailvalid(email, check_mx=False, verify=False):
+    """Indicate whether the given string is a valid email address
+    according to the 'addr-spec' portion of RFC 2822 (see section
+    3.4.1).  Parts of the spec that are marked obsolete are *not*
+    included in this test, and certain arcane constructions that
+    depend on circular definitions in the spec may not pass, but in
+    general this should correctly identify any email address likely
+    to be in use as of 2011.
+
+    For check the domain mx and verify email exits you must have the pyDNS package installed:
+    pip install pyDNS
+
+    Check if the host has SMTP Server: ('example@example.com',check_mx=True)
+    Check if the host has SMTP Server and the email really exists: ('example@example.com',verify=True)  <--not working sometimes?
+    """
+    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+    # modified from https://github.com/syrusakbary/validate_email
+    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+    # RFC 2822 - style email validation for Python
+    # (c) 2012 Syrus Akbary <me@syrusakbary.com>
+    # Extended from (c) 2011 Noel Bush <noel@aitools.org>
+    # for support of mx and user check (i.e., whether the mail server actually exists, user actually exists)
+    # This code is made available to you under the GNU LGPL v3.
+    #
+    # returns True or False to indicate whether a given address
+    # is valid according to the 'addr-spec' part of the specification
+    # given in RFC 2822.  Ideally, we would like to find this
+    # in some other library, already thoroughly tested and well-
+    # maintained.  The standard Python library email.utils
+    # contains a parse_addr() function, but it is not sufficient
+    # to detect many malformed addresses.
+    #
+    # This implementation aims to be faithful to the RFC, with the
+    # exception of a circular definition (see comments below), and
+    # with the omission of the pattern components marked as "obsolete".
+
+    import re
+    import smtplib
+    import socket
+
+    try:
+        import DNS
+        ServerError = DNS.ServerError
+    except:
+        DNS = None
+        class ServerError(Exception): pass
+    # All we are really doing is comparing the input string to one
+    # gigantic regular expression.  But building that regexp, and
+    # ensuring its correctness, is made much easier by assembling it
+    # from the "tokens" defined by the RFC.  Each of these tokens is
+    # tested in the accompanying unit test file.
+    #
+    # The section of RFC 2822 from which each pattern component is
+    # derived is given in an accompanying comment.
+    #
+    # (To make things simple, every string below is given as 'raw',
+    # even when it's not strictly necessary.  This way we don't forget
+    # when it is necessary.)
+    #
+    WSP = r'[ \t]'                                       # see 2.2.2. Structured Header Field Bodies
+    CRLF = r'(?:\r\n)'                                   # see 2.2.3. Long Header Fields
+    NO_WS_CTL = r'\x01-\x08\x0b\x0c\x0f-\x1f\x7f'        # see 3.2.1. Primitive Tokens
+    QUOTED_PAIR = r'(?:\\.)'                             # see 3.2.2. Quoted characters
+    FWS = r'(?:(?:' + WSP + r'*' + CRLF + r')?' + \
+                WSP + r'+)'                                    # see 3.2.3. Folding white space and comments
+    CTEXT = r'[' + NO_WS_CTL + \
+                    r'\x21-\x27\x2a-\x5b\x5d-\x7e]'              # see 3.2.3
+    CCONTENT = r'(?:' + CTEXT + r'|' + \
+                         QUOTED_PAIR + r')'                        # see 3.2.3 (NB: The RFC includes COMMENT here
+                                                                                                             # as well, but that would be circular.)
+    COMMENT = r'\((?:' + FWS + r'?' + CCONTENT + \
+                        r')*' + FWS + r'?\)'                       # see 3.2.3
+    CFWS = r'(?:' + FWS + r'?' + COMMENT + ')*(?:' + \
+                 FWS + '?' + COMMENT + '|' + FWS + ')'         # see 3.2.3
+    ATEXT = r'[\w!#$%&\'\*\+\-/=\?\^`\{\|\}~]'           # see 3.2.4. Atom
+    ATOM = CFWS + r'?' + ATEXT + r'+' + CFWS + r'?'      # see 3.2.4
+    DOT_ATOM_TEXT = ATEXT + r'+(?:\.' + ATEXT + r'+)*'   # see 3.2.4
+    DOT_ATOM = CFWS + r'?' + DOT_ATOM_TEXT + CFWS + r'?' # see 3.2.4
+    QTEXT = r'[' + NO_WS_CTL + \
+                    r'\x21\x23-\x5b\x5d-\x7e]'                   # see 3.2.5. Quoted strings
+    QCONTENT = r'(?:' + QTEXT + r'|' + \
+                         QUOTED_PAIR + r')'                        # see 3.2.5
+    QUOTED_STRING = CFWS + r'?' + r'"(?:' + FWS + \
+                                    r'?' + QCONTENT + r')*' + FWS + \
+                                    r'?' + r'"' + CFWS + r'?'
+    LOCAL_PART = r'(?:' + DOT_ATOM + r'|' + \
+                             QUOTED_STRING + r')'                    # see 3.4.1. Addr-spec specification
+    DTEXT = r'[' + NO_WS_CTL + r'\x21-\x5a\x5e-\x7e]'    # see 3.4.1
+    DCONTENT = r'(?:' + DTEXT + r'|' + \
+                         QUOTED_PAIR + r')'                        # see 3.4.1
+    DOMAIN_LITERAL = CFWS + r'?' + r'\[' + \
+                                     r'(?:' + FWS + r'?' + DCONTENT + \
+                                     r')*' + FWS + r'?\]' + CFWS + r'?'  # see 3.4.1
+    DOMAIN = r'(?:' + DOT_ATOM + r'|' + \
+                     DOMAIN_LITERAL + r')'                       # see 3.4.1
+    ADDR_SPEC = LOCAL_PART + r'@' + DOMAIN               # see 3.4.1
+
+    # A valid address will match exactly the 3.4.1 addr-spec.
+    VALID_ADDRESS_REGEXP = '^' + ADDR_SPEC + '$'
+
+    try:
+        assert re.match(VALID_ADDRESS_REGEXP, email) is not None
+        check_mx |= verify
+        if check_mx:
+            if not DNS: raise Exception('For check the mx records or check if the email exists you must have installed pyDNS python package')
+            DNS.DiscoverNameServers()
+            hostname = email[email.find('@')+1:]
+            mx_hosts = DNS.mxlookup(hostname)
+            for mx in mx_hosts:
+                try:
+                    smtp = smtplib.SMTP()
+                    smtp.connect(mx[1])
+                    if not verify: return True
+                    status, _ = smtp.helo()
+                    if status != 250: continue
+                    smtp.mail('')
+                    status, _ = smtp.rcpt(email)
+                    if status != 250: return False
+                    break
+                except smtplib.SMTPServerDisconnected: #Server not permits verify user
+                    break
+                except smtplib.SMTPConnectError:
+                    continue
+    except (AssertionError, ServerError):
+        return False
+    return True
+isEmailValid = isemailvalid
+IsEmailValid = isemailvalid
+
 # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 # debugging
 # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
